#!/bin/bash
#
# install.sh
#
# Copyright (C) 2011-2015 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.  


VERSION="2.2"

#-------------------------------------------------------------------------------
#				Script functions
#-------------------------------------------------------------------------------
usage()
{
	echo "Usage: ${0##*/} -d <directory> -u <user> -k <ssl_key> -c <ssl_cert> -a <ssl_ca_file> [-hV]"
	echo "-d <directory>            Warden client installation directory"
	echo "-u <user>                 owner of Warden client package (user for running detection scripts)"
	echo "-k <ssl_key>              path to SSL key"
	echo "-c <ssl_cert>             path to SSL certificate"
	echo "-a <ssl_ca>               path to SSL CA certificate"
	echo "-h                        print this help"
	echo "-V                        print script version number and exit"
	echo
	echo "Example: $ ./${0##*/} -d /opt/warden-client -u warden -k /etc/ssl/private/client.key -c /etc/ssl/certs/client.pem -a /etc/ssl/certs/tcs-ca-bundle.pem"
	echo
	echo "For more information about installation process, see README file (section Installation)."
	echo
	exit 0
}


version()
{
	echo "${0##*/} - current version is $VERSION"
	exit 0
}


err()
{
	echo "FAILED!"
	echo -n "Error: " && cat "$err"
	rm -f "$err"
	echo
	echo "Installation of $package_version package FAILED!!!"
	exit 1
}


errClean()
{
        echo "FAILED!"
        echo -n "Error: " && cat "$err"
        echo " -> Uninstalling client package ..."
        for file in $(<"$manifest")
        do
                rm -f "${basedir}/$file" 2> /dev/null
                directory="${file%/*}"
                rmdir --ignore-fail-on-non-empty "${basedir}/$directory" 2> /dev/null
        done
        rm -f "${basedir}/uninstall.sh" 2> /dev/null
        rm -f "$err"
        echo
        echo "Installation of $package_version package FAILED!!!"
        exit 1
}


paramsChck()
{
        if [ -z "$basedir" ]; then
                echo "Parameter -d <directory> is not set!"
                exit 1
        fi
	if [ -z "$user" ]; then
		echo "Parameter -u <user> is not set!"
		exit 1
	fi
        if [ -z "$key_file" ]; then
                echo "Parameter -k <ssl_key> is not set!"
                exit 1
        fi
        if [ -z "$cert_file" ]; then
                echo "Parameter -c <ssl_cert> is not set!"
                exit 1
        fi
        if [ -z "$ca_file" ]; then
                echo "Parameter -a <ssl_ca> is not set!"
                exit 1
        fi
}


#-------------------------------------------------------------------------------
#                               Installation functions
#-------------------------------------------------------------------------------

oldPackageChck()
{
        echo -n "Checking previous Warden client installation ... "
        old_package_version_file=$(find "$basedir" -name package_version)
        if [ -f "$old_package_version_file" ]; then
                old_package_version=$(<"$old_package_version_file")
                echo "FAILED!"
                echo
                echo "Package $old_package_version has been already installed!"
                echo "For update of Warden client package please use update.sh."
                exit 1
        fi
        echo "OK"
}


perlChck()
{
        echo -n "Checking Perl interpreter ... "
        if which perl 1> /dev/null; then
                echo "OK"
        else
                echo "FAILED!"
                echo
                echo "Perl interpreter is not installed!"
                exit 1
        fi
}


modulesChck()
{
        sed '/^use [A-Z]/!d; /Warden/d' $(find "${dirname}" -type f) 2>/dev/null | cut -f 2 -d " " | sed 's/;//' | sort -u | \
        while read module;
        do
                echo -n "Checking $module module ... "
                perl -e "use $module" 2> "$err" && echo "OK" || err
        done
}


createWardenDir()
{
        echo -n "Creating Warden client directory ... "
        test -d "$basedir" && test -w "$basedir" || mkdir -p "$basedir" 2> "$err" || err
        for file in $(<"$manifest")
        do
                dir="$basedir/${file%/*}"
                [[ ! -d "$dir" ]] && mkdir -p "$dir"
                cp "$file" "${basedir}/$file" 2> "$err" || errClean
        done
	mkdir "$var" 2> "$err" || errClean
        ln -s "${sh}/uninstall.sh" "$basedir"
        echo "OK"
}


doTemplate()
{
        vars=""
        while (($#)); do
                vars="'$1'=>'$2', $vars"
                shift 2
        done
        perl -e "my %repl=($vars); foreach my \$l (<>) {\$l =~ s/\$_/\$repl{\$_}/g for keys %repl; print \$l}"
}


createManifest()
{
        echo -n "Creating MANIFEST file ... "
        cd "$basedir" && find . -mindepth 2 -type f | sed 's/.\///' | sort > "${doc}/MANIFEST" && cd $OLDPWD 2> "$err" || errClean
        echo "OK"
}


setPermissions()
{
	echo -n "Setting of owner and permissions to installed package ... "
	for file in `ls "$basedir"`
	do
		chown -R "${user}": "$basedir/$file" 2> "$err" || errClean
	done
	chmod 644 "${etc}/package_version" || errClean
	if chmod 600 $client_conf; then
		echo "OK"
	else
		errClean
	fi
}



#-------------------------------------------------------------------------------
#				MAIN
#-------------------------------------------------------------------------------
# read input
while getopts "d:u:k:c:a:Vh" options; do
	case $options in
		d ) basedir=$OPTARG;;
		u ) user=$OPTARG;;
		k ) key_file=$OPTARG;;
		c ) cert_file=$OPTARG;;
		a ) ca_file=$OPTARG;;
		h ) usage;;
		V ) version;;
		* ) usage;;
	esac
done

# remove last char (slash) from name of directories
[[ "$basedir" == */ ]] && basedir="${basedir%?}"

# params test
paramsChck

# set variables
err="/tmp/warden-err"
dirname=${0%/*}
bin="${basedir}/bin"
lib="${basedir}/lib"
doc="${basedir}/doc"
etc="${basedir}/etc"
sh="${basedir}/sh"
var="${basedir}/var"
client_conf="${etc}/warden-client.conf"
client_conf_tmpl="${dirname}/etc/warden-client.conf.tmpl"
package_version=$(<"${dirname}/etc/package_version") 2> "$err" || err
manifest="${dirname}/doc/MANIFEST"


echo "Installation of Warden client package ($package_version) ..."
echo
echo "----------------------- Checking for Dependencies -----------------------"

# check if warden-client is installed
[[ ! -z "$basedir" ]] && [[ -d "$basedir" ]] && oldPackageChck

# Perl interpreter test
perlChck

# Perl modules test
modulesChck || exit 1


echo
echo "------------------------- Installation process --------------------------"

# make warden client directory
createWardenDir || exit 1

# create client configuration file
echo -n "Creating Warden client configuration file ... "
doTemplate \
        _CERTFILE_      "$cert_file" \
        _KEYFILE_       "$key_file" \
        _CAFILE_        "$ca_file" \
        < $client_conf_tmpl \
        > $client_conf && echo "OK" || errClean

# create MANIFEST file
createManifest

# set owner and permissions
setPermissions


echo
echo "------------------------- Summary ---------------------------------------"

echo "Please check configuration files:"
echo " - $client_conf"
echo
echo "Warden client directory: $basedir"
echo
echo "Installation of $package_version package was SUCCESSFUL!!!"
echo

# cleanup section
rm -f "$err"

exit 0
